@@ -1,4 +1,9 @@
+#[cfg(feature = "ratatui-termion")]
+use ratatui::termion;
+
 use crate::input::InputRequest;
+use crate::Input;
+use crate::StateChanged;
 use std::io::{Result, Write};
 use termion::cursor::Goto;
 use termion::event::{Event, Key};
@@ -9,17 +14,18 @@ use termion::style::NoInvert;
 pub fn to_input_request(evt: &Event) -> Option<InputRequest> {
     use InputRequest::*;
     match *evt {
-        Event::Key(Key::Backspace) => Some(DeletePrevChar),
+        Event::Key(Key::Backspace) | Event::Key(Key::Ctrl('h')) => Some(DeletePrevChar),
         Event::Key(Key::Delete) => Some(DeleteNextChar),
-        Event::Key(Key::Left) => Some(GoToPrevChar),
-        Event::Key(Key::Right) => Some(GoToNextChar),
+        Event::Key(Key::Left) | Event::Key(Key::Ctrl('b')) => Some(GoToPrevChar),
+        Event::Key(Key::Right) | Event::Key(Key::Ctrl('f')) => Some(GoToNextChar),
         // Event::Key(Key::Ctrl(Key::Left)) => Some(GoToPrevWord),
         // Event::Key(Key::Ctrl(Key::Right)) => Some(GoToNextWord),
         Event::Key(Key::Ctrl('u')) => Some(DeleteLine),
         Event::Key(Key::Ctrl('w')) => Some(DeletePrevWord),
         // Event::Key(Key::Ctrl(Key::Delete)) => Some(DeleteNextWord),
-        Event::Key(Key::Ctrl('a')) => Some(GoToStart),
-        Event::Key(Key::Ctrl('e')) => Some(GoToEnd),
+        Event::Key(Key::Ctrl('a')) | Event::Key(Key::Home) => Some(GoToStart),
+        Event::Key(Key::Ctrl('e')) | Event::Key(Key::End) => Some(GoToEnd),
+        Event::Key(Key::Char('\t')) => None,
         Event::Key(Key::Char(c)) => Some(InsertChar(c)),
         _ => None,
     }
@@ -62,3 +68,30 @@ pub fn write<W: Write>(
 
     Ok(())
 }
+
+/// Import this trait to implement `Input::handle_event()` for termion.
+pub trait EventHandler {
+    /// Handle termion event.
+    fn handle_event(&mut self, evt: &Event) -> Option<StateChanged>;
+}
+
+impl EventHandler for Input {
+    /// Handle termion event.
+    fn handle_event(&mut self, evt: &Event) -> Option<StateChanged> {
+        to_input_request(evt).and_then(|req| self.handle(req))
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn handle_tab() {
+        let evt = Event::Key(Key::Char('\t'));
+
+        let req = to_input_request(&evt);
+
+        assert!(req.is_none());
+    }
+}
