@@ -1,3 +1,20 @@
+//! Core logic for handling input.
+//!
+//! # Example: Without any backend
+//!
+//! ```
+//! use tui_input::{Input, InputRequest, StateChanged};
+//!
+//! let mut input: Input = "Hello Worl".into();
+//!
+//! let req = InputRequest::InsertChar('d');
+//! let resp = input.handle(req);
+//!
+//! assert_eq!(resp, Some(StateChanged { value: true, cursor: true }));
+//! assert_eq!(input.cursor(), 11);
+//! assert_eq!(input.to_string(), "Hello World");
+//! ```
+
 /// Input requests are used to change the input state.
 ///
 /// Different backends can be used to convert events into requests.
@@ -17,6 +34,7 @@ pub enum InputRequest {
     DeletePrevWord,
     DeleteNextWord,
     DeleteLine,
+    DeleteTillEnd,
 }
 
 #[derive(Debug, PartialOrd, PartialEq, Eq, Clone, Copy, Hash)]
@@ -35,11 +53,10 @@ pub type InputResponse = Option<StateChanged>;
 /// ```
 /// use tui_input::Input;
 ///
-/// let value = "Hello World".to_string();
-/// let input = Input::default().with_value(value);
+/// let input: Input = "Hello World".into();
 ///
-/// assert_eq!(input.value(), "Hello World");
 /// assert_eq!(input.cursor(), 11);
+/// assert_eq!(input.to_string(), "Hello World");
 /// ```
 #[derive(Default, Debug, Clone)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
@@ -49,6 +66,13 @@ pub struct Input {
 }
 
 impl Input {
+    /// Initialize a new instance with a given value
+    /// Cursor will be set to the given value's length.
+    pub fn new(value: String) -> Self {
+        let len = value.chars().count();
+        Self { value, cursor: len }
+    }
+
     /// Set the value manually.
     /// Cursor will be set to the given value's length.
     pub fn with_value(mut self, value: String) -> Self {
@@ -64,6 +88,19 @@ impl Input {
         self
     }
 
+    // Reset the cursor and value to default
+    pub fn reset(&mut self) {
+        self.cursor = Default::default();
+        self.value = Default::default();
+    }
+
+    // Reset the cursor and value to default, returning the previous value
+    pub fn value_and_reset(&mut self) -> String {
+        let val = self.value.clone();
+        self.reset();
+        val
+    }
+
     /// Handle request and emit response.
     pub fn handle(&mut self, req: InputRequest) -> InputResponse {
         use InputRequest::*;
@@ -159,10 +196,7 @@ impl Input {
                         .value
                         .chars()
                         .rev()
-                        .skip(
-                            self.value.chars().count().max(self.cursor)
-                                - self.cursor,
-                        )
+                        .skip(self.value.chars().count().max(self.cursor) - self.cursor)
                         .skip_while(|c| !c.is_alphanumeric())
                         .skip_while(|c| c.is_alphanumeric())
                         .count();
@@ -229,16 +263,12 @@ impl Input {
                         .value
                         .chars()
                         .rev()
-                        .skip(
-                            self.value.chars().count().max(self.cursor)
-                                - self.cursor,
-                        )
+                        .skip(self.value.chars().count().max(self.cursor) - self.cursor)
                         .skip_while(|c| !c.is_alphanumeric())
                         .skip_while(|c| c.is_alphanumeric())
                         .collect::<Vec<char>>();
                     let rev_len = rev.len();
-                    self.value =
-                        rev.into_iter().rev().chain(remaining).collect();
+                    self.value = rev.into_iter().rev().chain(remaining).collect();
                     self.cursor = rev_len;
                     Some(StateChanged {
                         value: true,
@@ -295,6 +325,14 @@ impl Input {
                     })
                 }
             }
+
+            DeleteTillEnd => {
+                self.value = self.value.chars().take(self.cursor).collect();
+                Some(StateChanged {
+                    value: true,
+                    cursor: false,
+                })
+            }
         }
     }
 
@@ -307,6 +345,65 @@ impl Input {
     pub fn cursor(&self) -> usize {
         self.cursor
     }
+
+    /// Get the current cursor position with account for multispace characters.
+    pub fn visual_cursor(&self) -> usize {
+        if self.cursor == 0 {
+            return 0;
+        }
+
+        // Safe, because the end index will always be within bounds
+        unicode_width::UnicodeWidthStr::width(unsafe {
+            self.value.get_unchecked(
+                0..self
+                    .value
+                    .char_indices()
+                    .nth(self.cursor)
+                    .map_or_else(|| self.value.len(), |(index, _)| index),
+            )
+        })
+    }
+
+    /// Get the scroll position with account for multispace characters.
+    pub fn visual_scroll(&self, width: usize) -> usize {
+        let scroll = (self.visual_cursor()).max(width) - width;
+        let mut uscroll = 0;
+        let mut chars = self.value().chars();
+
+        while uscroll < scroll {
+            match chars.next() {
+                Some(c) => {
+                    uscroll += unicode_width::UnicodeWidthChar::width(c).unwrap_or(0);
+                }
+                None => break,
+            }
+        }
+        uscroll
+    }
+}
+
+impl From<Input> for String {
+    fn from(input: Input) -> Self {
+        input.value
+    }
+}
+
+impl From<String> for Input {
+    fn from(value: String) -> Self {
+        Self::new(value)
+    }
+}
+
+impl From<&str> for Input {
+    fn from(value: &str) -> Self {
+        Self::new(value.into())
+    }
+}
+
+impl std::fmt::Display for Input {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        self.value.fmt(f)
+    }
 }
 
 #[cfg(test)]
@@ -316,9 +413,16 @@ mod tests {
 
     use super::*;
 
+    #[test]
+    fn format() {
+        let input: Input = TEXT.into();
+        println!("{}", input);
+        println!("{}", input);
+    }
+
     #[test]
     fn set_cursor() {
-        let mut input = Input::default().with_value(TEXT.into());
+        let mut input: Input = TEXT.into();
 
         let req = InputRequest::SetCursor(3);
         let resp = input.handle(req);
@@ -355,7 +459,7 @@ mod tests {
 
     #[test]
     fn insert_char() {
-        let mut input = Input::default().with_value(TEXT.into());
+        let mut input: Input = TEXT.into();
 
         let req = InputRequest::InsertChar('x');
         let resp = input.handle(req);
@@ -386,7 +490,7 @@ mod tests {
 
     #[test]
     fn go_to_prev_char() {
-        let mut input = Input::default().with_value(TEXT.into());
+        let mut input: Input = TEXT.into();
 
         let req = InputRequest::GoToPrevChar;
         let resp = input.handle(req);
@@ -414,7 +518,7 @@ mod tests {
 
     #[test]
     fn remove_unicode_chars() {
-        let mut input = Input::default().with_value("¡test¡".into());
+        let mut input: Input = "¡test¡".into();
 
         let req = InputRequest::DeletePrevChar;
         let resp = input.handle(req);
@@ -449,8 +553,7 @@ mod tests {
 
     #[test]
     fn insert_unicode_chars() {
-        let mut input =
-            Input::default().with_value("¡test¡".into()).with_cursor(5);
+        let mut input = Input::from("¡test¡").with_cursor(5);
 
         let req = InputRequest::InsertChar('☆');
         let resp = input.handle(req);
@@ -483,4 +586,12 @@ mod tests {
         assert_eq!(input.value(), "¡☆test☆¡");
         assert_eq!(input.cursor(), 2);
     }
+
+    #[test]
+    fn multispace_characters() {
+        let input: Input = "Ｈｅｌｌｏ, ｗｏｒｌｄ!".into();
+        assert_eq!(input.cursor(), 13);
+        assert_eq!(input.visual_cursor(), 23);
+        assert_eq!(input.visual_scroll(6), 18);
+    }
 }
